//If you want to Show all Message in Text mode,
//Unmark the following line.
//#define TEXT_MODE

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <memory.h>
#include "parser.h"

FILE* err=stdout;   //specify the FILE pointer for ERROR MESSAGE
FILE* show=stdout;  //specify the FILE pointer for Text_Show
FILE* out;          //specify the FILE pointer for output to file
char* IMAGE;        //IMAGE one dimension array BGR....
                    //program should free this memory
int Ximage,     //size of image in X 
    Yimage;     //size of image in Y


#define MAX_HUFFMAN_SUBTREE   50   // max size = MAX_HUFFMAN_SUBTREE * 256
unsigned int num_of_byte=0;

//get a 8-bit data from file pointer 'in'
inline unsigned int get8(FILE* in){
#ifdef TEXT_MODE
    num_of_byte++;
#endif
    return (unsigned int) fgetc(in);
}

//get a 16-bit data from file pointer 'in'
inline unsigned int get16(FILE* in){
    static unsigned int temp;
#ifdef TEXT_MODE
    num_of_byte+=2;
#endif
    temp=(unsigned int)fgetc(in);
    temp<<=8;
    return temp | (unsigned int) fgetc(in);
}

/********************************************************************
Huffman table for fast search: (HuffTab) 8-bit Look up table
  2-layer search architecture, 1st-layer represent 256 node (8 bits)
  if codeword-length > 8 bits, then 
      the entry of 1st-layer = (# of 2nd-layer table) | MSB
      and it is stored in the 2nd-layer
  Size of tables in each layer are 256. 
  HuffTab[*][*][0-256] is always the only 1st-layer table.

  An entry can be:
  (1) (# of 2nd-layer table) | MSB , for code length > 8 in 1st-layer
  (2) (Code length) << 8 | HuffVal
********************************************************************/
unsigned int HuffTab[4][2][MAX_HUFFMAN_SUBTREE * 256];

/* Build_HuffTab()
    Parameter:  t       table ID
                c       table class ( 0 for DC, 1 for AC )
                L[i]    # of codewords which length is i
                V[i][j] Huffman Value (length=i)
    Effect: 
        build up HuffTab[t][c] using L and V.
*/

void Build_HuffTab(unsigned int tab[],
				   unsigned int L[16], unsigned int V[16][200]){
    unsigned int current_table,i,j,n,table_used, temp;
	static unsigned int k;
	temp=256;
	k=0;
    for(i=0; i<8; i++){  // i+1 is Code length
        for(j=0; j<L[i]; j++){
            for(n=0; n < (temp>>(i+1)); n++){
                tab[k]=V[i][j] | ((i+1) << 8);
                k++;
            }
        }
    }
    for(i=1; k<256; i++, k++) tab[k]= i | MSB;
    if(i>50){
        fprintf(err,"ERROR: Huffman table out of memory!\n");
        exit(0);
    }
    table_used=i;
    current_table=1;
    k=0;
    for(i=8; i<16; i++){  // i+1 is Code length
        for(j=0; j<L[i]; j++){
            for(n=0; n < (temp>>(i-7)); n++){
                tab[current_table*256+k]=V[i][j] | ((i+1)<<8);
                k++;
            }
            if(k>=256){
                if(k>256){
                    fprintf(err,"ERROR: Huffman table error(1)!\n");
                    exit(0);
                }
                k=0; current_table++;
            }
        }
    }
}

/* Text_Show_HuffTab()
      Show content of HuffTab[t][c]
*/
void Text_Show_HuffTab(int t, int c){
    int i,j, n=1, CODE=0, length, value, prev_length;
    char str[33];
    prev_length=0;
    fprintf(show,"<HUFFMAN TABLE: ID=%i Class=%i>\n", t,c);
    for(j=0; j<256;){
        if(HuffTab[t][c][j] & MSB){
            n++; j++;
        }else{
            length=HuffTab[t][c][j] >> 8;
            value= HuffTab[t][c][j] & 0xFF;
            CODE <<= (length-prev_length);
            if(length){
                int tmp, a;
                fprintf(show,"\tVal=%i\tCODE=",value);
                itoa(CODE,str,2);
                for(tmp=0; str[tmp]!='\0'; tmp++);
                for(a=0; a < (length-tmp); a++) fprintf(show,"0");
                fprintf(show,"%s",str);
                for(a=0; a < (20-length); a++) fprintf(show," ");
                fprintf(show,"Length=%i\n",length);
            }
            prev_length=length;
            CODE++;
            for(; (HuffTab[t][c][j]&0xFF) == value; j++);
        }
    }
    for(i=1; i<n; i++){
        for(j=0; j<256;){
            if(HuffTab[t][c][256*i+j] & MSB){
                n++; j++;
            }else{
                length=HuffTab[t][c][i*256+j] >> 8;
                value= HuffTab[t][c][i*256+j] & 0xFF;
                CODE <<= (length-prev_length);
                if(length){
                    int tmp, a;
                    fprintf(show,"\tVal=%i\tCODE=",value);
                    itoa(CODE,str,2);
                    for(tmp=0; str[tmp]!='\0'; tmp++);
                    for(a=0; a < (length-tmp); a++) fprintf(show,"0");
                    fprintf(show,"%s",str);
                    for(a=0; a < (20-length); a++) fprintf(show," ");
                    fprintf(show,"Length=%i\n",length);
                }
                prev_length=length;
                CODE++;
                for(; (HuffTab[t][c][i*256+j]&0xFF) == value; j++);
            }
        }
    }
    fprintf(show,"</HUFFMAN TABLE>\n");
}

/* HuffmanValue(): 
    return: Huffman Value of table
            0xFF?? if it receives a MARKER
    Parameter:  table   HuffTab[x][y] (ex) HuffmanValue(HuffTab[1][0],...)
                temp    temp storage for remainded bits
                index   index to bit of temp
                in      FILE pointer
    Effect:
        temp  store new remainded bits
        index change to new index
        in    change to new position
    NOTE:
      Initial by   temp=0; index=0;
    NOTE: (explain temp and index)
      temp: is always in the form at calling time or returning time
       |  byte 4  |  byte 3  |  byte 2  |  byte 1  |
       |     0    |     0    | 00000000 | 00000??? |  if not a MARKER
                                               ^index=3 (from 0 to 15)
                                               321    
    NOTE (marker and marker_index):
      If get a MARKER from 'in', marker=the low-byte of the MARKER
        and marker_index=9
      If marker_index=9 then index is always > 8, or HuffmanValue()
        will not be called.
*/
unsigned int marker;
int marker_index=0;

inline int HuffmanValue(unsigned int table[],
                        unsigned int& temp,int& index, FILE* in){
    static unsigned int code, input ,mask=0xFFFF;
    if(index<8){
        temp <<= 8;
        input = get8(in);
        if(input==0xFF){
            marker=get8(in);
            if(marker) marker_index=9;
        }
        temp |= input;
    } else index -= 8;
    code=table[temp >> index];
    if(code&MSB){
        if(marker_index){ marker_index=0; return 0xFF00|marker;}
        temp &= (mask >> (16-index));
        temp <<= 8;
        input = get8(in);
        if(input==0xFF){
            marker=get8(in);
            if(marker) marker_index=9;
        }
        temp |= input;
        code=table[(code&0xFF)*256 + (temp >> index)];
        index+=8;
    }
    index += 8 - (code>>8);
    if(index<0) fprintf(err,"\nindex=%i temp=%X code=%X in HuffmanValue()\n",
                       index,temp,code);
    if(index < marker_index){ marker_index=0; return 0xFF00|marker;}
    temp &= ( mask >> (16-index) );
    return code&0xFF;
}

//get n-bit signed data from file 'in'
// temp is defined as before
// return signed integer or 0x00FF??00 if it sees a MARKER
inline int getn(FILE* in, int n, unsigned int& temp, int& index){
    static int result, one=1, n_one=-1;
    static unsigned int mask=0xFFFF, input;
    if(n==0) return 0;
    index -= n;
    if(index>=0){
        if(index < marker_index){
            marker_index=0; 
            return (0xFF00|marker) << 8;
        }
        result = temp>>index;
        temp &= ( mask >> (16-index) );
    }else{
        temp <<= 8;
        input = get8(in);
        if(input==0xFF){
            marker=get8(in);
            if(marker) marker_index=9;
        }
        temp |= input;
        index += 8;
        if(index < 0){
            if(marker_index){
                marker_index=0;
                return (0xFF00|marker) << 8;
            }
            temp <<= 8;
            input = get8(in);
            if(input==0xFF){
                marker=get8(in);
                if(marker) marker_index=9;
            }
            temp |= input;
            index += 8;
        }
        if(index<0) fprintf(err,"index=%i in getn()\n", index);
        if(index<marker_index){
            marker_index=0;
            return (0xFF00|marker) << 8;
        }
        result = temp>>index;
        temp &= ( mask >> (16-index) );
    }
    if( result < (one<<(n-1)) )
        result += (n_one << n) + 1;
    return result;
}

/******************************************************************

  Decode MCU

    DU[i][j][8][8]     the j-th data unit of component i.

******************************************************************/

int DU[10][4][64];   //at most 10 data units in a MCU
                       //at most 4 data units in one component 


inline int YUV_to_R(int& y, int& u, int& v){
    int temp = y+128+((91881*v)>>16);
    if(temp>255) return 255;
    if(temp<0) return 0;
    return temp;
}

inline int YUV_to_G(int& y, int& u, int& v){
    int temp = y+128-((22554*u+46802*v)>>16);
    if(temp>255) return 255;
    if(temp<0) return 0;
    return temp;
}

inline int YUV_to_B(int& y, int& u, int& v){
    int temp = y+128+((116130*u)>>16);
    if(temp>255) return 255;
    if(temp<0) return 0;
    return temp;
}

#define YUV_TO_R(y,u,v)  ( y+128+((91881*v)>>16) )

#define YUV_TO_G(y,u,v) ( y+128-((22554*u+46802*v)>>16) )

#define YUV_TO_B(y,u,v) ( y+128+((116130*u)>>16) )

void YUV_to_BGR(int& y,int& u,int& v,char* BGR_IMAGE){
    int NEW_Y=y+128;
    int tempB,tempG,tempR;
    if(y<-128) NEW_Y=0;
    tempB=NEW_Y+((116130*u)>>16);
    if(tempB<0) *(BGR_IMAGE++)=0; 
    else if(tempB>255) *(BGR_IMAGE++)='\xFF';
    else *(BGR_IMAGE++)=tempB;

    tempG=NEW_Y-((22554*u+46802*v)>>16);
    if(tempG<0) *(BGR_IMAGE++)=0; 
    else if(tempG>255) *(BGR_IMAGE++)='\xFF';
    else *(BGR_IMAGE++)=tempG;
    
    tempR=NEW_Y+((91881*v)>>16);
    if(tempR<0) *(BGR_IMAGE)=0;
    else if(tempR>255) *(BGR_IMAGE)='\xFF';
    else *(BGR_IMAGE)=tempR;
}


/* output()
    x, y should be the starting point of MCU when calling output(..)
      it means output() should set x,y for the next MCU at the end.
*/

void Parser_class::output(void){
    static int x=0, y=0, num=0; // the begin point of MCU
    int temp_x, temp_8y, temp;
	int k=0;
    int *DU10, *DU20;
	int YsizeMinus1;
	YsizeMinus1=Ysize-1;
    DU10=DU[1][0]; DU20=DU[2][0];
#ifdef TEXT_MODE
    fprintf(show,"<MCU %i>\n", num);
#endif
    num++;
    for(unsigned int i=0; i<YV; i++){
        for(unsigned int j=0; j<YH; j++){
			temp_8y=i*32;
			temp_x=temp=j*4;
            for(int l=0; l<64; l++){
                if(x<Xsize && y<Ysize){
                   YUV_to_BGR(DU[0][k][l],DU10[temp_8y+temp_x],DU20[temp_8y+temp_x],
					          IMAGE+(x*3+(YsizeMinus1-y)*Xsize*3));
                }
                x++;
				if( (x%YH)==0 ) temp_x++;
                if( (x%8)==0 ){ 
					y++; x-=8;
					temp_x = temp;
					if( (y%YV)==0 ) temp_8y+=8;
				}
            }
            k++;
            x+=8;
            y-=8;
        }
        x -= YH*8;
        y += 8;
    }
    x += YH*8;
    y -= YV*8;
    if(x>=Xsize){
        y+=YV*8;
		x=0;
    }
}

inline void level_shift(int* du, unsigned int& P){
    int i;
    if(P==8){
        for(i=0; i<64; i++)
            du[i] += 128;
    }else if(P==12){
        for(i=0; i<64; i++)
            du[i] += 2048;
    }else
        fprintf(err,"ERROR: Precision=%i\n", P);
}

/* decode_MCU()
     return 0       if correctly decoded
            0xFF??  if it sees a MARKER
*/

int IDCT_Source[64];

unsigned int 
Parser_class::decode_MCU(FILE* in, int* PrevDC,
                         unsigned int& temp, int& index)
{
	static const int IDCT_P[64] =
	{
	   0,   5,  40,  16,  45,   2,   7,  42,
	  21,  56,   8,  61,  18,  47,   1,   4,
	  41,  23,  58,  13,  32,  24,  37,  10,
	  63,  17,  44,   3,   6,  43,  20,  57,
	  15,  34,  29,  48,  53,  26,  39,   9,
	  60,  19,  46,  22,  59,  12,  33,  31,
	  50,  55,  25,  36,  11,  62,  14,  35,
	  28,  49,  52,  27,  38,  30,  51,  54	
	} ;
    unsigned int value,*actab,*dctab;
    int *qtab,Cs;
    
    for(Cs=0; Cs<nComp; Cs++){
        qtab=qTab[Cs]; actab=acTab[Cs]; dctab=dcTab[Cs];
	    for(int i=0 ; i<nBlock[Cs]; i++){
            memset(IDCT_Source, 0, sizeof(int[64]));
		    value = HuffmanValue(dctab,temp,index,in);
            if(value>=0xFF00)
                return value;
		    PrevDC[Cs] = IDCT_Source[0] = PrevDC[Cs] + getn(in,value,temp,index);
		    IDCT_Source[0] *= qtab[0];
		    for(int j=1 ; j<64 ; j++){
			    value = HuffmanValue(actab,temp,index,in);
                if(value>=0xFF00)
                    return value;
			    j += (value>>4) ;
			    if( (value&0x0F) == 0){
				    if( (value>>4) == 0) break ;
			    }else {
				    IDCT_Source[IDCT_P[j]] = 
                        getn(in,value&0x0F,temp,index)*qtab[j];
			    }
		    }
		    ScaleIDCT(DU[Cs][i]) ;
	    }
    }
    return 0;
}

// in-place operation
void EnhanceQuantizationTable( int *qtab )
{
	static int table[] = {
		0,	1, 5,	6,	14,	15,	27,	28, 
		2,	4,	7,	13,	16,	26,	29,	42,
		3,	8,	12,	17,	25,	30,	41,	43,
		9,	11,	18,	24,	31,	40,	44,	53,
		10, 19, 23, 32, 39, 45, 52, 54,
		20, 22, 33, 38, 46, 51, 55, 60,
		21, 34, 37, 47, 50, 56, 59, 61,
		35, 36, 48, 49, 57, 58, 62, 63
	};
	int i ;
	for( i = 0 ; i < 8 ; i ++ )
	{
		qtab[table[0*8+i]] *= 90  ;
		qtab[table[4*8+i]] *= 90  ;
		qtab[table[2*8+i]] *= 118 ;
		qtab[table[6*8+i]] *= 49  ;
		qtab[table[5*8+i]] *= 71  ;
		qtab[table[1*8+i]] *= 126 ;
		qtab[table[7*8+i]] *= 25  ;
		qtab[table[3*8+i]] *= 106 ;
	}
	for( i = 0 ; i < 8 ; i ++ )
	{
		qtab[table[0+8*i]] *= 90  ;
		qtab[table[4+8*i]] *= 90  ;
		qtab[table[2+8*i]] *= 118 ;
		qtab[table[6+8*i]] *= 49  ;
		qtab[table[5+8*i]] *= 71  ;
		qtab[table[1+8*i]] *= 126 ;
		qtab[table[7+8*i]] *= 25  ;
		qtab[table[3+8*i]] *= 106 ;
	}
	for( i = 0 ; i < 64 ; i++ ) {
		qtab[i] >>= 6 ;
	}
}

// out-of-place operation
// input: IDCT_Source
// output: matrix
void Parser_class::ScaleIDCT(int* matrix)
{
	static int p[8][8] ;
	int t0 , t1 , t2 , t3, i ;
	int src0, src1, src2, src3, src4, src5, src6, src7;
	int det0, det1, det2, det3, det4, det5, det6, det7;

	for( i = 0 ; i < 8 ; i++ )
	{
		src0 = IDCT_Source[0*8+i] ;
		src1 = IDCT_Source[1*8+i] ;
		src2 = IDCT_Source[2*8+i] - IDCT_Source[3*8+i] ;
		src3 = IDCT_Source[3*8+i] + IDCT_Source[2*8+i] ;
		src4 = IDCT_Source[4*8+i] - IDCT_Source[7*8+i] ;
		src6 = IDCT_Source[5*8+i] - IDCT_Source[6*8+i] ;
		t0   = IDCT_Source[5*8+i] + IDCT_Source[6*8+i] ;
		t1   = IDCT_Source[4*8+i] + IDCT_Source[7*8+i] ;
		src5 = t0 - t1 ;
		src7 = t0 + t1 ;
		//
		det4 =-src4 * 480 - src6 * 192 ;
		det5 = src5 * 384 ;
		det6 = src6 * 480 - src4 * 192 ;
		det7 = src7 * 256 ;
		t0   = src0 * 256 ;
		t1   = src1 * 256 ;
		t2   = src2 * 384 ;
		t3   = src3 * 256 ;
		det3 = t3      ;
		det0 = t0 + t1 ;
		det1 = t0 - t1 ;
		det2 = t2 - t3 ;
		//
		src0 = det0 + det3 ;
		src1 = det1 + det2 ;
		src2 = det1 - det2 ;
		src3 = det0 - det3 ;
		src4 = det6 - det4 - det5  - det7 ;
		src5 = det5 - det6 + det7 ;
		src6 = det6 - det7 ;
		src7 = det7 ;
		//
		p[0][i] = ( src0 + src7 +(1<<12))>>13 ;
		p[1][i] = ( src1 + src6 +(1<<12))>>13 ;
		p[2][i] = ( src2 + src5 +(1<<12))>>13 ;
		p[3][i] = ( src3 + src4 +(1<<12))>>13 ;
		p[4][i] = ( src3 - src4 +(1<<12))>>13 ;
		p[5][i] = ( src2 - src5 +(1<<12))>>13 ;
		p[6][i] = ( src1 - src6 +(1<<12))>>13 ;
		p[7][i] = ( src0 - src7 +(1<<12))>>13 ;
	}
	//
	for( i = 0 ; i < 8 ; i++ )
	{
		src0 = p[i][0] ;
		src1 = p[i][1] ;
		src2 = p[i][2] - p[i][3] ;
		src3 = p[i][3] + p[i][2] ;
		src4 = p[i][4] - p[i][7] ;
		src6 = p[i][5] - p[i][6] ;
		t0   = p[i][5] + p[i][6] ;
		t1   = p[i][4] + p[i][7] ;
		src5 = t0 - t1 ;
		src7 = t0 + t1 ;
		//
		det4 =-src4 * 480 - src6 * 192 ;
		det5 = src5 * 384 ;
		det6 = src6 * 480 - src4 * 192 ;
		det7 = src7 * 256 ;
		t0   = src0 * 256 ;
		t1   = src1 * 256 ;
		t2   = src2 * 384 ;
		t3   = src3 * 256 ;
		det3 = t3      ;
		det0 = t0 + t1 ;
		det1 = t0 - t1 ;
		det2 = t2 - t3 ;
		//
		src0 = det0 + det3 ;
		src1 = det1 + det2 ;
		src2 = det1 - det2 ;
		src3 = det0 - det3 ;
		src4 = det6 - det4 - det5  - det7 ;
		src5 = det5 - det6 + det7 ;
		src6 = det6 - det7 ;
		src7 = det7 ;
		//
		matrix[0] = ( src0 + src7 +(1<<12))>>13 ;
		matrix[1] = ( src1 + src6 +(1<<12))>>13 ;
		matrix[2] = ( src2 + src5 +(1<<12))>>13 ;
		matrix[3] = ( src3 + src4 +(1<<12))>>13 ;
		matrix[4] = ( src3 - src4 +(1<<12))>>13 ;
		matrix[5] = ( src2 - src5 +(1<<12))>>13 ;
		matrix[6] = ( src1 - src6 +(1<<12))>>13 ;
		matrix[7] = ( src0 - src7 +(1<<12))>>13 ;
		matrix+=8;
	}
}	


/*******************************************
    implementation of classes
********************************************/
int Frame_header_class::get(FILE* in, unsigned int sof){
    unsigned int i, temp, count=0, c;
    SOF=sof;
    Lf=get16(in); count+=2;
    P=get8(in);   count++;
    Y=get16(in);  count+=2;
    Yimage=Y;
    X=get16(in);  count+=2;
    Ximage=X;
    Nf=get8(in);  count++;
    Comp = new Component_spec_class[Nf+1];
    for(i=1; i<=Nf; i++){
        if(count>Lf){
            fprintf(err,"ERROR: fram format error\n");
            exit(0);
        }
        c=get8(in);          count++;
        if(c>=Lf){
            fprintf(err,"ERROR: fram format error [c>=Lf]\n");
            exit(0);
        }
        Comp[c].C=c;
        temp=get8(in);       count++;
        Comp[c].H=temp>>4;
        Comp[c].V=temp&0x0F;
        Comp[c].Tq=get8(in); count++;
    }
    if(count!=Lf){
        fprintf(err,"ERROR: frame format error [Lf!=count]\n");
        exit(0);
    }
#ifdef TEXT_MODE
    text_show();
#endif
    return 1;
}

void Frame_header_class::text_show(void){
    unsigned int i;
    fprintf(show,"<FRAM HEADER>\n");
    fprintf(show,"  SOF=%X  Lf=%i  P=%i  Y=%i  X=%i  Nf=%i\n",
           SOF,Lf,P,Y,X,Nf);
    fprintf(show,"  Components:\n");
    for(i=0; i<=Nf; i++)
        fprintf(show,"    [C%i]  C=%i  H=%i  V=%i  Tq=%i\n",i,
               Comp[i].C, Comp[i].H, Comp[i].V, Comp[i].Tq);
    fprintf(show,"</FRAM HEADER>\n");
}

int Scan_header_class::get(FILE*in){
    unsigned int i,temp,count=0;
    Ls=get16(in);   count+=2;
    Ns=get8(in);    count++;
    Comp = new Scan_component_spec_class[Ns];
    for(i=0; i<Ns; i++){
        if(count>Ls){
            fprintf(err,"ERROR: scan header format error\n");
            exit(0);
        }
        Comp[i].Cs=get8(in); count++;
        temp=get8(in);       count++;
        Comp[i].Td=temp>>4;
        Comp[i].Ta=temp&0x0F;
    }
    Ss=get8(in);    count++;
    Se=get8(in);    count++;
    temp=get8(in);  count++;
    Ah=temp>>4;
    Al=temp&0x0F;
    if(count!=Ls){
        fprintf(err,"ERROR: scan header format error [count!=Ns]\n");
#ifdef TEXT_MODE
		text_show();
#endif
		exit(0);
    }
#ifdef TEXT_MODE
    text_show();
#endif
    return 1;
}

void Scan_header_class::text_show(void){
    unsigned int i;
    fprintf(show,"<SCAN HEADER>\n");
    fprintf(show,"  Ls=%i  Ns=%i  Ss=%i  Se=%i  Ah=%i  Al=%i\n",
           Ls, Ns, Ss, Se, Ah, Al);
    fprintf(show,"  Components:\n");
    for(i=0; i<Ns; i++)
        fprintf(show,"    Cs=%i  Td=%i  Ta=%i\n", 
               Comp[i].Cs, Comp[i].Td, Comp[i].Ta);
    fprintf(show,"</SCAN HEADER>\n");
}

int Quantization_table_class::get(FILE*in){
    unsigned int i,count=0, temp, t;
    Lq=get16(in); count+=2;
    while(count<Lq){
        temp=get8(in);  count++;
        t=temp&0x0F;
        if(t>3){
            fprintf(err,"ERROR: Quantization table ID > 3\n");
            exit(0);
        }
        Pq[t]=temp>>4;
        if(Pq[t]==0) Pq[t]=8;
        else if(Pq[t]==1) Pq[t]=16;
        else{
            fprintf(err,"ERROR: Quantization table precision error\n");
            exit(0);
        }
        Tq[t]=1;
        if(Pq[t]==8){
            for(i=0; i<64; i++){
                if(count>Lq){
                    fprintf(err,"ERROR: Quantization table format error\n");
                    exit(0);
                }
                Q[t][i]=get8(in); count++;
            }
            EnhanceQuantizationTable(Q[t]);
        }else{
            for(i=0; i<64; i++){
                if(count>Lq){
                    fprintf(err,"ERROR: Quantization table format error\n");
                    exit(0);
                }
                Q[t][i]=get16(in); count+=2;
            }
            EnhanceQuantizationTable(Q[t]);
        }
    }
    if(count!=Lq){
        fprintf(err,"ERROR: Quantization table error [count!=Lq]\n");
        exit(0);
    }
#ifdef TEXT_MODE
    text_show();
#endif
    return 1;
}

void Quantization_table_class::text_show(void){
    int i,j;
    fprintf(show,"<QUANTIZATION TABLE>  Lq=%i\n" ,Lq);
    for(i=0; i<4; i++){
        if(Tq[i]!=1) continue;
        fprintf(show,"  Table:%i  Precision=%i\n", i, Pq[i]);
        for(j=0; j<64; j++){
            fprintf(show,"\t%i",Q[i][j]);
            if(j%8==7) fprintf(show,"\n");
        }
    }
    fprintf(show,"</QUANTIZATION TABLE>\n");
}

int Huffman_table_class::get(FILE* in){
    unsigned int i,j,temp,count=0,t,c;
    Lh=get16(in);    count+=2;
    while(count<Lh){
        temp=get8(in);  count++;
        t=temp&0x0F;
        if(t>3){
            fprintf(err,"ERROR: Huffman table ID > 3\n");
            exit(0);
        }
        c=temp>>4;
        if(c>2){
            fprintf(err,"ERROR: Huffman table [Table class > 2 ]\n");
            exit(0);
        }
        Th[t]=1; Tc[t][c]=1;
        for(i=0; i<16; i++){
            L[t][c][i]=get8(in);  count++;
        }
        for(i=0; i<16; i++)
            for(j=0; j<L[t][c][i]; j++){
                if(count>Lh){
                    fprintf(err,"ERROR: Huffman table format error [count>Lh]\n");
                    exit(0);
                }
                V[t][c][i][j]=get8(in); count++;
            }
    }
    if(count!=Lh){
        fprintf(err,"ERROR: Huffman table format error [count!=Lf]\n");
        exit(0);
    }
#ifdef TEXT_MODE
    text_show();
#endif
    for(i=0; i<4; i++)
        for(j=0; j<2; j++)
            if(Tc[i][j]){
                Build_HuffTab(HuffTab[i][j],L[i][j],V[i][j]);
#ifdef TEXT_MODE
                Text_Show_HuffTab(i,j);
#endif
            }
    return 1;
}

void Huffman_table_class::text_show(void){
    unsigned int i,j,k,l;
    fprintf(show,"<HUFFMAN TABLE>  Lh=%i\n" ,Lh);
    for(i=0; i<4; i++)
        for(j=0; j<2; j++){
            if(Tc[i][j]!=1) continue;
            fprintf(show,"  Table:%i  class=%i\n", i, j);
            for(k=0; k<16; k++)
                for(l=0; l<L[i][j][k]; l++)
                    fprintf(show,"    BITS=%i  HUFFVAL=%i\n",k+1,V[i][j][k][l]);
                
        }
    fprintf(show,"</HUFFMAN TABLE>\n");
}

int Restart_interval_class::get(FILE* in){
    Lr=get16(in);
    if(Lr!=4){
        fprintf(err,"ERROR: Restart interval length != 4\n");
        exit(0);
    }
    Ri=get16(in);
#ifdef TEXT_MODE
    text_show();
#endif
    return 1;
}

void Restart_interval_class::text_show(void){
    fprintf(show,"<RESTART INTERVAL Ri=%i>\n", Ri);
}

int Define_number_class::get(FILE* in){
    Ld=get16(in);
    if(Ld!=4){
        fprintf(err,"ERROR: Define number format error [Ld!=4]\n");
        exit(0);
    }
    NL=get16(in);
#ifdef TEXT_MODE
    text_show();
#endif
    return 1;
}

void Define_number_class::text_show(void){
    fprintf(show,"<DEFINE NUMBER  NL=%i>\n", NL);
}

int Comment_class::get(FILE* in){
    unsigned int count=0, i;
    Lc=get16(in);   count+=2;
    Cm = new char[Lc];
    for(i=0; count<Lc; i++){
        Cm[i]=get8(in); count++;
    }
    Cm[i]='\0';
#ifdef TEXT_MODE
    text_show();
#endif
    return 1;
}

void Comment_class::text_show(void){
    fprintf(show,"<Comment>\n%s\n</Comment>\n",Cm);
}

int Application_class::get(FILE* in){
    unsigned int count=0;
    Lp=get16(in);   count+=2;
    while(count<Lp){
        get8(in); count++;
    }
#ifdef TEXT_SHOW
    text_show();
#endif
    return 1;
}

void Application_class::text_show(void){
    fprintf(show,"<Application  Lp=%i>\n", Lp);
}

void Parser_class::decode(FILE* in){
    unsigned int current,m,i,scan_num=0,RST_num;
    int PRED[10];
	if(in==NULL) return;
    current=get16(in);
    if(current!=0xFFD8){  //SOI
        fprintf(err,"This file is not an JPEG file!\n");
        return;
    }
    current=get16(in);
    while(current>>4 != 0x0FFC){   //SOF 0~15
        switch(current){
        case 0xFFC4:  //DHT
            HT.get(in); break;
        case 0xFFCC:  //DAC
            fprintf(err,"Program doesn't support arithmetic coding. (format error)\n");
            return;
        case 0xFFDB:
            QT.get(in); break;
        case 0xFFDD:
            RI.get(in); break;
        case 0xFFE0: case 0xFFE1: case 0xFFE2: case 0xFFE3:
        case 0xFFE4: case 0xFFE5: case 0xFFE6: case 0xFFE7:
        case 0xFFE8: case 0xFFE9: case 0xFFEA: case 0xFFEB:
        case 0xFFEC: case 0xFFED: case 0xFFEE: case 0xFFEF:
            APP.get(in); break;
        case 0xFFFE:
            COM.get(in); break;
        default:
            if(current>>8 != 0xFF){
                fprintf(err,"ERROR: format error! (Parser.decode)\n");
                exit(0);
            }
        }
        current=get16(in);
    }
    if(current<0xFFC0 || current >0xFFC7){
        fprintf(err,"ERROR: could not headle arithmetic code!\n");
        exit(0);
    }
    
    Frame_header.get(in,current);
    current=get16(in);

    IMAGE = new char[Frame_header.X * Frame_header.Y * 3];

Start_of_a_Scan:

    while(current != 0x0FFDA){   //SOS
        switch(current){
        case 0xFFC4:  //DHT
            HT.get(in); break;
        case 0xFFCC:  //DAC
            fprintf(err,"Program doesn't support arithmetic coding. (format error)\n");
            exit(0);
        case 0xFFDB:
            QT.get(in); break;
        case 0xFFDD:
            RI.get(in); break;
        case 0xFFE0: case 0xFFE1: case 0xFFE2: case 0xFFE3:
        case 0xFFE4: case 0xFFE5: case 0xFFE6: case 0xFFE7:
        case 0xFFE8: case 0xFFE9: case 0xFFEA: case 0xFFEB:
        case 0xFFEC: case 0xFFED: case 0xFFEE: case 0xFFEF:
            APP.get(in); break;
        case 0xFFFE:
            COM.get(in); break;
        default:
            if(current>>8 != 0xFF){
                fprintf(err,"ERROR: format error! (Parser.decode)\n");
                exit(0);
            }
        }
        current=get16(in);
    }

    Scan_header.get(in);
    nComp=(int)Scan_header.Ns;
    for(i=0; i<nComp; i++){
        int CompN=Scan_header.Comp[i].Cs;
        qTab[i]=QT.Q[Frame_header.Comp[CompN].Tq];
        nBlock[i]=Frame_header.Comp[CompN].V * Frame_header.Comp[CompN].H;
        dcTab[i]=HuffTab[Scan_header.Comp[i].Td][0];
        acTab[i]=HuffTab[Scan_header.Comp[i].Ta][1];
    }
    YH=Frame_header.Comp[1].H; YV=Frame_header.Comp[1].V;
    Xsize=Frame_header.X; Ysize=Frame_header.Y;
    
    scan_num++;
    m=0;

    for(RST_num=0;;RST_num++){  //Decode one scan
        unsigned int MCU_num;
        unsigned int temp;  // to store remainded bits
        int index;
        temp=0;
        index=0;
        for(i=0; i<10; i++) PRED[i]=0;
        if(RI.Ri==0){
            current=decode_MCU(in,PRED,temp,index);
                                            // 0: correctly decoded
                                            // otherwise: MARKER
            while(current==0){
                m++;
                output();
                current=decode_MCU(in,PRED,temp,index);
            }
            break;  //current=MARKER
        }
        for(MCU_num=0; MCU_num<RI.Ri; MCU_num++){
            current=decode_MCU(in,PRED,temp,index);
            output();
            //fprintf(show,"%i ",MCU_num);
            if(current!=0) break;
        }
#ifdef TEXT_MODE
        fprintf(show,"\ncurrent1=%X temp=%X index=%i marker=%X marker_index=%i\n",
                current, temp, index, marker, marker_index);
#endif
        if(current==0){
            if(marker_index){
                current=(0xFF00|marker);
                marker_index=0;
            }else current=get16(in);
#ifdef TEXT_MODE
            fprintf(show,"current2=%X\n", current);
#endif
        }
        if(current>=0xFFD0 && current<=0xFFD7){
#ifdef TEXT_MODE
            fprintf(show,"RSTn=%X\n", current);
#endif
        }else break; //current=MARKER
#ifdef TEXT_MODE
        fprintf(show,"--End Of Restart inteval %i--\n", RST_num);
#endif
    }
    if(current==0xFFDC && scan_num==1){ //DNL
        NL.get(in);
        current=get16(in);
    }
    if(current!=0xFFD9) goto Start_of_a_Scan;

//End_of_all_Scans
#ifdef TEXT_MODE
	fprintf(show,"Total byte=%i\n", num_of_byte);
#endif
}

//--------------------------------------------------------------------
#ifdef TEXT_MODE
void main(void){
    Parser_class JPGparser;
    FILE* in;
    char filename[100]="test1.jpg";
	clock_t begin, end;
    err=stdout;
	show=stdout;
    out=fopen("output.tst", "wb");
    printf("input file: ");
    scanf("%s", filename);
    in=fopen(filename, "rb");
	begin=clock();
    JPGparser.decode(in);
	end=clock();
	printf("Total time=%d ms\n", end-begin);

    for(int i=0; i<Ximage*3*Yimage; i+=3)
        fprintf(out,"%c%c%c",IMAGE[i+2],IMAGE[i+1],IMAGE[i]);

	if(in!=NULL)
		fclose(in);
}
#endif